########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Lib/__init__.py,v 1.16 2004/07/22 11:39:16 mbrown Exp $
"""
Module providing common utilities for many 4Suite components,
as well as for general use.

Copyright 2004 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

from Ft import FtException

class UriException(FtException):
    """
    Exceptions used by the Uri module, and possibly others.
    """

    INVALID_BASE_URI = 100

    #RELATIVE_DOCUMENT_URI = 110
    RELATIVE_BASE_URI = 111
    OPAQUE_BASE_URI = 112

    NON_FILE_URI = 120
    UNIX_REMOTE_HOST_FILE_URI = 121

    RESOURCE_ERROR = 130

    SCHEME_REQUIRED = 200 # for SchemeRegistryResolver

    INVALID_PUBLIC_ID_URN = 300

    UNSUPPORTED_PLATFORM = 1000

    def __init__(self, errorCode, *args, **kwargs):
        import MessageSource
        self.params = args or (kwargs,)
        self.errorCode = errorCode
        messages = MessageSource.URI
        msgargs = args or kwargs
        self.message = messages[errorCode] % msgargs
        Exception.__init__(self, self.message, args)
        return


def Truncate(text, length):
    """
    Returns text truncated to length, with "..." appended if truncation
    was necessary.
    """
    if len(text) > length:
        return text[:length] + '...'
    else:
        return text[:length]


def Wrap(text, width):
    """
    A word-wrap function that preserves existing line breaks
    and most spaces in the text. Expects that existing line
    breaks are posix newlines (\n).
    """
    # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/148061
    return reduce(lambda line, word, width=width: '%s%s%s' %
                  (line,
                   ' \n'[(len(line[line.rfind('\n')+1:])
                         + len(word.split('\n',1)[0]
                              ) >= width)],
                   word),
                  text.split(' ')
                 )
