########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Fetch/Container.py,v 1.14 2004/07/11 09:47:26 mbrown Exp $
"""
Implementation of '4ss fetch container' command

Copyright 2004 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Fetches a directory-like listing for a Container in the 4Suite repository.
Columns are: Internet Media Type, size, date last modified, relative name."""

import os, sys, time

from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes

def Run(options, args):

    repo = None
    try:
        repo = CommandUtil.GetRepository(options, '4ss.fetch.container')
        try:
            container = repo.fetchResource(args.get('name', '/'))
            if container.isResourceType(ResourceTypes.ResourceType.CONTAINER):
                sty = repo.fetchResource('/ftss/data/cl-container.xslt')
                print container.applyXslt([sty])[0]
            else:
                #FIXME: use FtServerClientException?
                sys.stderr.write("The resource '%s' is not a container.\n" % container.getPath())
                sys.stderr.flush()
        finally:
            try:
                if repo is not None:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n' + '\n')


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('container',
                          'List contents of a Container in the 4Suite repository',
                          '/ftss/data',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.OptionalArgument('name',
                                                                  'name (repo path) of container to view (default: /)',
                                                                  str),
                                       ],

                          fileName = __file__,
                          )
    return cmd
