########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Fetch/Document.py,v 1.10 2004/07/11 09:47:26 mbrown Exp $
"""
Implementation of '4ss fetch document' command

Copyright 2004 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Fetches an XML Document from the 4Suite repository."""

import sys, warnings
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil

def Run(options,args):
    output = sys.stdout
    quiet = options.get('quiet', 0)

    try:
        repo = CommandUtil.GetRepository(options,'4ss.fetch.document')
        try:
            doc = repo.fetchResource(
                args['name'])
            if not quiet:
                sys.stderr.write("Document %s (%s)\n" % (doc.getPath(), doc.getImt()))
                sys.stderr.write("-"*40+"\n")
                sys.stderr.flush()
            sys.stdout.write(doc.getContent())
            if not quiet:
                sys.stderr.write("\n")
                sys.stderr.flush()
        finally:
            try:
                repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')
        return


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('document',
                          'Fetches an XML Document from the 4Suite repository.',
                          '/ftss/demos/index.html',
                          __doc__,
                          function=Run,
                          arguments=[Arguments.RequiredArgument('name',
                                                                'The name (repo path) of document to fetch',
                                                                str),
                                       ],
                          options=Options.Options([
                              Options.Option('q', 'quiet', "Don't preface output with informational header"),
                              ]),
                          fileName=__file__,
                          )
    return cmd


