########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Fetch/RawFile.py,v 1.9 2004/07/11 09:47:26 mbrown Exp $
"""
Implementation of '4ss fetch rawfile' command

Copyright 2004 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Fetches a Raw File from the 4Suite repository.
Since most resource types inherit from Raw File, this command can be used to get the content of nearly any file."""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil


def Run(options,args):
    output = sys.stdout
    quiet = options.get('quiet')
    try:
        repo = CommandUtil.GetRepository(options,'4ss.fetch.rawfile')
        try:
            doc = repo.fetchResource(args['name'])

            if not quiet:
                sys.stderr.write("Raw File %s (%s)\n" % (args['name'],doc.getImt()))
                sys.stderr.write("-"*40+"\n")
                sys.stderr.flush()
            output.write(doc.getContent())

        finally:
            repo.txRollback()
    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')
        return


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('rawfile',
                          'Fetches a Raw File from the 4Suite repository',
                          '/ftss/data/4SSPowered.jpg > localfile.jpg',
                          __doc__,
                          function=Run,
                          arguments = [Arguments.RequiredArgument('name',
                                                                  'The name (repo path) of the resource to fetch.',
                                                                  str),
                                       ],
                          options=Options.Options([
                              Options.Option('q', 'quiet', "Don't preface output with informational header"),
                              ]),
                          fileName=__file__,
                          )
    return cmd

