########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Fetch/UserData.py,v 1.8 2004/07/11 09:47:26 mbrown Exp $
"""
Implementation of '4ss fetch userdata' command

Copyright 2004 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Summarizes the user data for a User in the 4Suite repository.
Summarizes the keys and values. If no keys are specified, then all of the keys associated with the user are fetched."""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes

def Run(options, args):
    quiet = options.get('quiet', 0)
    try:
        name = args['userName']
        repo = CommandUtil.GetRepository(options,'4ss.fetch.userdata')
        try:
            user = repo.fetchUserOrGroupByName(name)
            if user is not None and user.isResourceType(ResourceTypes.ResourceType.USER):

                if args['key']:
                    userData = {}
                    for key in args['key']:
                        userData[key] = user.get(key)
                else:
                    userData = user.getUserData()

                names = userData.keys()
                names.sort()

                if not quiet:
                    sys.stderr.write("User: %s\n" % args['userName'])
                    sys.stderr.write("-"*30+"\n")
                    sys.stderr.flush()
                for name in names:
                    print "%s: %s" % (name,userData[name])

            else:
                #FIXME: use FtServerClientException?
                sys.stderr.write("User %s does not exist.\n" % name)
                sys.stderr.flush()
        finally:
            repo.txRollback()
    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')



        return


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('userdata',
                          "Summarizes the user data for a User in the 4Suite repository.""",
                          'root',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('userName',
                                                                  'the name of the user whose data to fetch',
                                                                  str),
                                       Arguments.ZeroOrMoreArgument('key',
                                                                    'the key (name) of a name-value pair; if none, all data for the user will be returned',
                                                                    str),
                                       ],
                          options=Options.Options([
                              Options.Option('q', 'quiet', "Don't preface output with informational header"),
                              ]),
                          fileName = __file__,
                          )
    return cmd

