########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Rdf/Complete.py,v 1.15 2003/10/07 15:44:31 mbrown Exp $
"""
Implementation of '4ss rdf complete' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Perform an RDF "complete": a query that returns, by \
default, all RDF statements from the combined system and user models."""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import Util


def Run(options, args):

    ns_map = None
    try:
        repo = CommandUtil.GetRepository(options,'4ss.rdf.complete')
        try:
            model = repo.getModel()
            res = model.complete(options.get('subject'),
                                 options.get('predicate'),
                                 options.get('object'),
                                 statementUri = options.get('uri'),
                                 scope = options.get('scope'))
            ns_map = Util.GetUserNsMappings(repo) #This is only needed for --xml
        finally:
            try:
                repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')
        return

    if options.get('xml'):
        #This get rather convoluted
        from Ft.Rdf.Drivers import Memory
        from Ft.Rdf import Model
        from Ft.Rdf.Serializers import Dom
        from Ft.Xml.Domlette import PrettyPrint
        mem = Memory.DbAdapter('')
        mem.begin()
        mod = Model.Model(mem)
        mod.add(res)
        ser = Dom.Serializer()
        res = ser.serialize(mod, nsMap=ns_map)
        PrettyPrint(res)
    elif options.get('flat-model'):
        print "Statements"
        print "----------"
        print
        for s in res:
            print s.subject, s.predicate, s.object, s.uri
    else:
        scopes = {}
        for s in res:
            if not scopes.has_key(s.scope):
                scopes[s.scope] = []
            scopes[s.scope].append(s)
        for k, v in scopes.items():
            heading = 'Scope: %s' % k
            print heading
            print "-" * len(heading)
            print
            for s in v:
                print (u'%s %s %s%s%s' % (s.subject, s.predicate,
                                          Display(s.object, OBJ_LEN),
                                          ' ' * (not not s.uri),
                                          s.uri)).encode('utf-8')
            print


OBJ_LEN = 50
def Display(text, length):
    if len(text) > length:
        result = text[:length] + '...'
    else:
        # why not just result=text?
        result = text[:length]
    result = result.replace("\n", "\\n")
    result = result.replace("\r", "\\r")
    return result


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('complete',
                          'Perform an RDF complete',
                          '--subject=urn:uuid1234456',
                          __doc__,
                          function = Run,
                          arguments = [
                                       ],
                          options = Options.Options([Options.Option(None,'subject=','The subject for the complete'),
                                                     Options.Option(None,'predicate=','The predicate for the complete'),
                                                     Options.Option(None,'object=','The object for the complete'),
                                                     Options.Option(None,'uri=','The URI of the statement to fetch'),
                                                     Options.Option(None,'scope=','The scope of statements to fetch.'),
                                                     Options.Option('x', 'xml', 'Generate Serialized RDF in XML format.'),
                                                     Options.Option(None, 'flat-model', 'Do not show the results sorted by scope.'),
                                      ]),
                          fileName = __file__,
                          )
    return cmd


