########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Xml/Lib/_4xupdate.py,v 1.13 2004/05/28 06:51:03 uogbuji Exp $
"""
4XUpdate command-line application

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """4XUpdate command-line application"""

import os, sys, cStringIO
from Ft.Xml import __version__
from Ft.Xml import XUpdate
from Ft.Xml import Domlette, InputSource
from Ft.Lib import Uri

def Run(options,args):
    dump = options.get('inplace')
    outfile = options.get('out-file')
    source_uri = args['source-uri']
    xupdate_uri = args['xupdate-uri']

    reader = Domlette.NonvalidatingReader
    xureader = XUpdate.Reader()

    processor = XUpdate.Processor()

    if os.path.isfile(source_uri):
        source_uri = Uri.OsPathToUri(source_uri, attemptAbsolute=1)
    source = reader.parseUri(source_uri)
    isrc = InputSource.DefaultFactory.fromUri(xupdate_uri)
    xupdate = xureader.fromSrc(isrc)

    processor.execute(source, xupdate)

    if outfile:
        stream = open(outfile, 'w')
    else:
        stream = sys.stdout
    Domlette.Print(source, stream=stream)

    return


from Ft.Lib.CommandLine import Options, CommandLineApp, Arguments, Command

class XUpdateCommandLineApp(CommandLineApp.CommandLineApp):

    def __init__(self):
        CommandLineApp.CommandLineApp.__init__(
            self,
            '4xupdate',
            '4XUpdate version %s' % __version__,
            __doc__,
            [],
            ourOptions = Options.Options([Options.Option('i',
                                                         'in-place',
                                                         'Update the source file in place',
                                                         ),
                                          Options.Option('o',
                                                         'out-file=',
                                                         'Write the result to the given output file',
                                                         ),
                                          ]),

            enableShowCommands = 0
            )

        self.function = Run
        self.arguments = [Arguments.RequiredArgument('source-uri',
                                                     'The URI of the source XML document. Note: if you use "-" as the name '\
                                                     'of the source document, the source will instead be read from standard input. "-" cannot be specified as the source URI if the "-i" option is used.',
                                                     str),
                          Arguments.RequiredArgument('xupdate-uri',
                                                     'The URI of the XUpdate instruction file. Note: if you use "-" as the name '\
                                                     'of the source document, the source will instead be read from standard input. '\
                                                     'The "-" option can only be used for the source or the XUpdate argument, not both.',
                                                     str),
                          ]


    def validate_arguments(self,args):
        return Command.Command.validate_arguments(self,args)
