########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Xml/Xslt/_4xslt.py,v 1.41 2004/03/27 04:28:10 mbrown Exp $
"""
Command-line invocation of the 4XSLT processor

Copyright 2004 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

import re, os, sys, traceback, cPickle, time
from Ft.Lib import Uri, UriException
from Ft.Xml import __version__
from Ft.Xml import XPath, InputSource
from Ft.Xml.Xslt import Processor, XsltException

__doc__ = """4XSLT command-line application"""

g_paramBindingPattern = re.compile(r"([\d\D_\.\-]*:?[\d\D_\.\-]+)=(.*)")

from Ft import MAX_PYTHON_RECURSION_DEPTH
sys.setrecursionlimit(MAX_PYTHON_RECURSION_DEPTH)

def Run(options, args):

    if options.has_key('version'):
        print '4XSLT, from 4Suite %s' % __version__
        return

    if not args.has_key('source-uri'):
        print 'A source URI argument is required. See "4xslt -h" for usage info.'
        return

    # Lots of ways to specify alternative base URIs for includes/imports.
    # We combine them into one big list, giving preference to command line
    # options first.
    alt_sty_uris = options.get('alt-sty-uri', [])
    if type(alt_sty_uris) != type([]):
        alt_sty_uris = [alt_sty_uris]

    alt_sty_paths = options.get('alt-sty-path', [])
    if type(alt_sty_paths) != type([]):
        alt_sty_paths = [alt_sty_paths]
    more_sty_uris = [Uri.OsPathToUri(path, attemptAbsolute=1) for path in alt_sty_paths]
    alt_sty_uris.extend(more_sty_uris)

    if os.environ.has_key('XSLTINCLUDE'):
        more_sty_uris = [
            Uri.OsPathToUri(path, attemptAbsolute=1)
            for path in os.environ["XSLTINCLUDE"].split(os.pathsep)
            ]
        alt_sty_uris.extend(more_sty_uris)

    del more_sty_uris

    xinclude = not options.has_key('noxinclude')
    ignore_pis = options.has_key('ignore')
    if options.has_key('compile'):
        output_flags = "wb"
    else:
        output_flags = "w"

    out_file = options.has_key('outfile') \
               and open(options['outfile'], output_flags) \
               or sys.stdout

    validate_flag = options.has_key('validate')
    checktime = options.has_key('time')

    # Do the file checking here; the rest of 4XSLT is URI based
    source = args['source-uri']
    if os.path.isfile(source):
        source = Uri.OsPathToUri(source, attemptAbsolute=1)

    stylesheets = []
    for uri in args['stylesheet-uri']:
        if os.path.isfile(uri):
            uri = Uri.OsPathToUri(uri, attemptAbsolute=1)
        stylesheets.append(uri)

    if options.has_key('compile') or options.has_key('reflex'):
        stylesheets = [source]

    stacktrace_on_error = options.has_key('stacktrace-on-error')

    if options.has_key('trace'):
        if not (options.has_key('outfile') or \
                options.has_key('trace-file')):
            raise SystemExit("When using the trace option, you must specify"
                             " an output file for the trace info"
                             " (--trace-file) or the transformation results"
                             " (-o or --outfile).")

        trace_file_name= options.get('trace-file', None)
        if trace_file_name:
            trace_file_name = os.path.abspath(trace_file_name)

        out_file_name = options.get('outfile', None)
        if out_file_name:
            out_file_name = os.path.abspath(out_file_name)

        if trace_file_name == out_file_name:
            raise SystemExit("The trace file and the output file cannot be"
                             " the same file.")

        trace_file = options.has_key('trace-file') and \
                     open(options['trace-file'], 'w') or sys.stdout

        from Ft.Xml.Xslt import ExtendedProcessingElements, StylesheetHandler
        processor = ExtendedProcessingElements.ExtendedProcessor(stylesheetAltUris=alt_sty_uris)
        processor._4xslt_trace = 1
        processor._4xslt_traceStream = trace_file
        StylesheetHandler._ELEMENT_MAPPING = ExtendedProcessingElements.GetMappings()
    else:
        processor = Processor.Processor(stylesheetAltUris=alt_sty_uris)

    processor.mediaPref = options.get('media', None)

    #Attempt to register EXT modules
    moduleList = os.environ.get("EXTMODULES")
    if moduleList:
        processor.registerExtensionModules(moduleList.split(":"))

    #Apply top-level parameter overrides
    defs = options.get('define', [])
    if type(defs) != type([]):
        defs = [defs]

    top_level_params = {}
    for d in defs:
        match = g_paramBindingPattern.match(d)
        top_level_params[match.group(1)] = match.group(2)

    from Ft.Xml import Domlette

    if validate_flag:
        reader = Domlette.ValidatingReader
    else:
        reader = Domlette.NonvalidatingReader

    #Customize message behavior
    if options.has_key('no-messages'):
        processor.messageControl(1)
    else:
        processor.messageControl(0)
        if options.has_key('msg-prefix'):
            prefix = options['msg-prefix']
            prefix = prefix.replace('\\n', '\n')
            prefix = prefix.replace('\\r', '\r')
            prefix = prefix.replace('\\t', '\t')
            processor.msgPrefix = prefix
        if options.has_key('msg-suffix'):
            suffix = options['msg-suffix']
            suffix = suffix.replace('\\n', '\n')
            suffix = suffix.replace('\\r', '\r')
            suffix = suffix.replace('\\t', '\t')
            processor.msgSuffix = suffix

    try:
        processor.setDocumentReader(reader)
        for sty in stylesheets:
            isrc = None
            try:
                isrc = InputSource.DefaultFactory.fromUri(sty)
            except (OSError, UriException), error:
                for uri in processor._stylesheetAltUris:
                    try:
                        new_sty = Uri.BASIC_RESOLVER.normalize(sty, path)
                        isrc = InputSource.DefaultFactory.fromUri(new_sty)
                        break
                    except (OSError, UriException):
                        pass
                if not isrc:
                    raise error
            except:
                traceback.print_exc(1000, sys.stderr)
            if options.has_key('instant'):
                istyle = cPickle.load(isrc)
                processor.appendStylesheetInstance(istyle.stylesheet,
                                                   refUri=isrc.uri)
            else:
                processor.appendStylesheet(isrc)

        if options.has_key('compile'):
            cPickle.dump(processor.stylesheet.root, out_file, 1)
            return

        if source == '-':
            isrc = InputSource.InputSource(sys.stdin,'.')
        else:
            isrc = InputSource.DefaultFactory.fromUri(source)

        if checktime:
            start = time.time()
        processor.run(isrc, ignore_pis, topLevelParams=top_level_params,
                      outputStream=out_file)
        if checktime:
            end = time.time()
    except XsltException, error:
        if stacktrace_on_error:
            traceback.print_exc(1000, sys.stderr)
        raise SystemExit(''.join([c.encode(sys.getdefaultencoding(), 'ignore')
                                  or "&#%d;" % ord(c) for c in error.message]))
    except (XPath.RuntimeException, XPath.CompiletimeException), error:
        if stacktrace_on_error:
            traceback.print_exc(1000, sys.stderr)
        raise SystemExit(''.join([c.encode(sys.getdefaultencoding(), 'ignore')
                                  or "&#%d;" % ord(c) for c in error.message]))
    except Exception, error:
        if stacktrace_on_error:
            traceback.print_exc(1000, sys.stderr)
            msg = ''
        else:
            msg = 'An unexpected error occurred while processing.\n' + \
                  'The error was: %s: %s\n' % (str(sys.exc_type)[11:], str(error)) + \
                  'Use the -e (stacktrace-on-error) option for a full stack trace.'
        raise SystemExit(''.join([c.encode(sys.getdefaultencoding(), 'ignore')
                                  or "&#%d;" % ord(c) for c in msg]))

    if out_file.isatty():
        out_file.flush()
        sys.stderr.write('\n')
    out_file.close()

    if checktime:
        sys.stderr.write("Transformation time: %dms\n" % (1000 * (end - start)))

    return


from Ft.Lib.CommandLine import Options, CommandLineApp, Arguments, Command


class XsltCommandLineApp(CommandLineApp.CommandLineApp):
    def __init__(self):
        CommandLineApp.CommandLineApp.__init__(
            self,
            '4xslt',
            '4XSLT version %s' % __version__,
            __doc__,
            [],
            ourOptions = Options.Options([Options.Option('V',
                                                         'version',
                                                         'Display program version and exit',
                                                         ),
                                          Options.Option('v',
                                                         'validate',
                                                         'Validate the input file as it is being parsed',
                                                         ),
                                          Options.Option('i',
                                                         'ignore',
                                                         'Ignore <?xml-stylesheet ...?> instructions'
                                                         ),
                                          Options.Option(None,
                                                         'media=MEDIA',
                                                         'Set media to honor in xml-stylesheet PIs'
                                                         ),
                                          Options.Option('D',
                                                         'define=NAME=VALUE',
                                                         'Bind a top-level parameter'
                                                         ),
                                          Options.Option('I',
                                                         'alt-sty-path=PATH',
                                                         "Same as --alt-sty-uri but uses OS path"
                                                         ),
                                          Options.Option(None,
                                                         'alt-sty-uri=URI',
                                                         "Define an add'l base URI for imports & includes"
                                                         ),
                                          Options.Option('o',
                                                         'outfile=FILE',
                                                         'Direct transformation output to FILE '\
                                                         '(file will be overwritten if it exists)'
                                                         ),
                                          Options.Option('e',
                                                         'stacktrace-on-error',
                                                         'Display a stack trace when an error occurs',
                                                         ),
                                          Options.Option(None,
                                                         'noxinclude',
                                                         'Do not expand XIncludes in source or stylesheet'
                                                         ),
                                          Options.Option(None,
                                                         'trace',
                                                         'Send execution trace output to stdout or file',
                                                         ),
                                          Options.Option(None,
                                                         'trace-file=FILE',
                                                         'Set trace file for execution trace output',
                                                         ),
                                          Options.Option(None,
                                                         'reflex',
                                                         'The stylesheet is also the source document.  Useful for cases where the stylesheet doesn\'t really use the source.'
                                                         ),
                                          Options.Option(None,
                                                         'compile',
                                                         'Compile an instant stylesheet.  The result is written to stdout, unless -o is used.'
                                                         ),
                                          Options.Option(None,
                                                         'instant',
                                                         'The stylesheet passed in is an "instant" (compiled) stylesheet.  Only one stylesheet can be specified with this option.'
                                                         ),
                                          Options.Option(None,
                                                         'time',
                                                         'Display the elapsed transformation time on stderr'
                                                         ),
                                          Options.Option(None,
                                                         'msg-prefix=STRING',
                                                         'Prepend string to xsl:message output'
                                                         ),
                                          Options.Option(None,
                                                         'msg-suffix=STRING',
                                                         'Append string to xsl:message output'
                                                         ),
                                          Options.Option(None,
                                                         'no-messages',
                                                         'Suppress xsl:message output and warnings'
                                                         ),
                                          ]),
            enableShowCommands = 0
            )

        self.function = Run
        self.arguments = [Arguments.OptionalArgument(
            'source-uri',
            'The URI of the source XML document. Required unless running "4xslt -V",'\
            'or with --reflex or --compile.'\
            ' Note: if you use "-" as the name of the source document, the source will'\
            ' instead be read from standard input.',
            str),
                          Arguments.ZeroOrMoreArgument('stylesheet-uri',
                                                      'The URI(s) of the stylesheets to apply to the source XML.',
                                                      str),
                          ]

    def validate_arguments(self,args):
        return Command.Command.validate_arguments(self,args)


def Register():
    return XsltCommandLineApp()

