########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Xml/_4xml.py,v 1.18 2004/06/08 19:48:34 mbrown Exp $
"""
4XML command-line tool

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

import re, os, sys, codecs
from Ft.Xml import __version__
from Ft.Xml import InputSource
from Ft.Lib import Uri

__doc__ = """4XML command-line application"""


def Run(options, args):
    if options.has_key('version'):
        print '4XML, from 4Suite %s' % __version__
        return

    if not args.has_key('source-uri'):
        print 'A source URI argument is required. See "4xml -h" for usage info.'
        return

    xinclude = not options.has_key('noxinclude')
    noserialize = options.has_key('noserialize')
    output = options.has_key('outfile') and open(options['outfile'], 'w') or sys.stdout
    validate_flag = options.has_key('validate')
    pretty = options.has_key('pretty')
    as_html = options.has_key('html')
    encoding = options.get('encoding', 'utf-8')
    input_encoding = options.get('input-encoding', None)
    rng_file = options.get('rng')

    # Do the file checking here; the rest of 4XML is uri based
    source = args['source-uri']
    if os.path.isfile(source):
        source = Uri.OsPathToUri(source, attemptAbsolute=1)

    if rng_file and os.path.isfile(rng_file):
        rng_file = Uri.OsPathToUri(rng_file, attemptAbsolute=1)

    from Ft.Xml import Domlette

    if validate_flag:
        reader = Domlette.ValidatingReader
    else:
        reader = Domlette.NonvalidatingReader

    try:
        if source == '-':
            isrc = InputSource.InputSource(
                sys.stdin, '.', processIncludes=xinclude,
                encoding=input_encoding)
        else:
            isrc = InputSource.DefaultFactory.fromUri(
                source, processIncludes=xinclude,
                encoding=input_encoding)
        doc = reader.parse(isrc)
        if rng_file:
            try:
                from Ft.Xml.Xvif import RelaxNgValidator, RngInvalid
            except ImportError:
                raise SystemExit("Missing RELAX-NG support library.  It is "
                                 "available in the thirdparty package.")
            rng_isrc = InputSource.DefaultFactory.fromUri(
                rng_file, processIncludes=xinclude,
                encoding=input_encoding)
            validator = RelaxNgValidator(rng_isrc)
            result = validator.validateNode(doc)
            if not result.nullable():
                raise RngInvalid(result)
        if not noserialize:
            from Ft.Xml.Domlette import Print, PrettyPrint
            if pretty:
                PrettyPrint(doc, output, encoding, as_html)
            else:
                Print(doc, output, encoding, as_html)
    except Exception, e:
        import traceback; traceback.print_exc(1000, sys.stderr)
        raise

    output.close()


from Ft.Lib.CommandLine import Options, CommandLineApp, Arguments, Command


class XmlCommandLineApp(CommandLineApp.CommandLineApp):
    def __init__(self):
        CommandLineApp.CommandLineApp.__init__(
            self,
            '4xml',
            '4XML version %s' % __version__,
            __doc__,
            [],
            ourOptions = Options.Options([Options.Option('V',
                                                         'version',
                                                         'Display program version and exit',
                                                         ),
                                          Options.Option('v',
                                                         'validate',
                                                         'DTD validate the input file as it is being parsed',
                                                         ),
                                          Options.Option('e',
                                                         'encoding=ENC',
                                                         'The encoding to use for output',
                                                         ),
                                          Options.Option(None,
                                                         'input-encoding=ENC',
                                                         'The encoding to assume for input',
                                                         ),
                                          Options.Option('o',
                                                         'outfile=FILE',
                                                         'Direct transformation output to FILE '\
                                                         '(file will be overwritten if it exists)'
                                                         ),
                                          Options.Option('p',
                                                         'pretty',
                                                         'Pretty-print the result',
                                                         ),
                                          Options.Option('n',
                                                         'noserialize',
                                                         "Don't serialize; just parse",
                                                         ),
                                          Options.Option(None,
                                                         'html',
                                                         'Use HTML mode when pretty-printing (emit XHTML as HTML)',
                                                         ),
                                          Options.Option(None,
                                                         'noxinclude',
                                                         'Do not expand XIncludes in source or stylesheet'
                                                         ),
                                          Options.Option(None,
                                                         'rng=FILE',
                                                         'Apply RELAX NG from the given file (technically RNG with XVIF features)'
                                                         ),
                                          ]),
            enableShowCommands = 0
            )

        self.function = Run
        self.arguments = [Arguments.OptionalArgument(
            'source-uri',
            'The URI of the XML document to parse.  Required unless running "4xml -V".'\
            ' Use the URI "-" to read from standard input.',
            str),
                          ]


    def validate_arguments(self,args):
        return Command.Command.validate_arguments(self,args)


def Register():
    return XmlCommandLineApp()

